/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::File System
 * Copyright (c) 2004-2016 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    fs_debug.h
 * Purpose: File System Debug Configuration
 *----------------------------------------------------------------------------*/

#ifndef __FS_DEBUG_H
#define __FS_DEBUG_H

#include <stdarg.h>
#include <stdint.h>

#include "EventRecorder.h"              // Keil::Compiler:Event Recorder
#include "RTE_Components.h"             // Component selection
#include "rl_fs_lib.h"

#if defined(__CC_ARM)
#pragma O3
#endif

/* File system debug configuration */
#if defined(FS_DEBUG_ENABLE) && (FS_DEBUG_ENABLE != 0)
uint16_t const fs_dbg_cfg = (FS_DEBUG_COM    <<  0) |
                            (FS_DEBUG_FAT    <<  2) |
                            (FS_DEBUG_EFS    <<  4) |
                            (FS_DEBUG_IOC    <<  6) |
                            (FS_DEBUG_NFTL   <<  8) |
                            (FS_DEBUG_NAND   << 10) |
                            (FS_DEBUG_MCI    << 12) |
                            (FS_DEBUG_SPI    << 14) ;
#else
uint16_t const fs_dbg_cfg = 0;
#endif

/**
  File System debug event output function
 
  \param[in]  n    number of va_list arguments
  \param[in]  id   message id
  \param[in]  ap   va_list arguments
*/
void fs_dbg_send_evt (uint32_t n, uint16_t id, va_list ap) {
  uint32_t p0, p1, p2, p3, p4, p5, p6, p7;

  p0 = va_arg (ap, uint32_t);
  p1 = va_arg (ap, uint32_t);
  p2 = va_arg (ap, uint32_t);
  p3 = va_arg (ap, uint32_t);
  p4 = va_arg (ap, uint32_t);
  p5 = va_arg (ap, uint32_t);
  p6 = va_arg (ap, uint32_t);
  p7 = va_arg (ap, uint32_t);

  /* Send event */
  switch (n) {
    case 0: Event_Record0 (id);
      break;

    case 1: Event_Record1 (id, p0);
      break;

    case 2: Event_Record2 (id, p0, p1);
      break;

    case 3: Event_Record3 (id, p0, p1, p2);
      break;

    case 4: Event_Record4 (id, p0, p1, p2, p3);
      break;

    case 5: Event_Record5 (id, p0, p1, p2, p3, p4);
      break;

    case 6: Event_Record6 (id, p0, p1, p2, p3, p4, p5);
      break;

    case 7: Event_Record7 (id, p0, p1, p2, p3, p4, p5, p6);
      break;
    
    case 8: Event_Record8 (id, p0, p1, p2, p3, p4, p5, p6, p7);
      break;
  }
}


/**
  File System debug buffer output function

  \param[in]  id   message id
  \param[in]  ap   va_list arguments
*/
void fs_dbg_send_buf (uint16_t id, va_list ap) {
  const uint8_t *data;
  uint32_t len;

  data = va_arg (ap, const uint8_t *);
  len  = va_arg (ap, uint32_t);

  /* Send buffer */
  Event_Record (id, data, len);
}

#endif /* __FS_DEBUG_H */
