/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2016 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    usb_compiler.h
 * Purpose: USB Library Compiler specific definitions
 * Rev.:    V6.8.0
 *----------------------------------------------------------------------------*/

#ifndef __USB_COMPILER_H__
#define __USB_COMPILER_H__

#include "stdint.h"

#if   defined ( __CC_ARM )                                            /* ARM Compiler 4/5 */
  #define __ASM                     __asm
  #define __REV                     __rev
  #define __NOP                     __nop
  #define __INLINE                  __inline
  #define __STATIC_INLINE           static __inline
  #define __NO_RETURN               __declspec(noreturn)
  #define __USED                    __attribute__((used))
  #define __WEAK                    __attribute__((weak))
  #define __PACKED                  __attribute__((packed))
  #define __UNALIGNED_UINT32(x)     (*((__packed uint32_t *)(x)))
  #define __AT_ADDR(x)              __attribute__((at(x)))
  #define __SECTION__(x)            __attribute__((section(#x)))
  #define __SECTION(x)              __SECTION__(x)

  #define osThreadDef_(...)         osThreadDef(__VA_ARGS__)
  #define osTimerDef_(...)          osTimerDef(__VA_ARGS__)
  #define osMutexDef_(name)         osMutexDef(name)
  #define osSemaphoreDef_(name)     osSemaphoreDef(name)

#elif defined (__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)       /* ARM Compiler 6 */
  #define __ASM                     __asm
  #define __REV                     __builtin_bswap32
  #define __NOP(x)                  __asm("nop")
  #define __INLINE                  __inline
  #define __STATIC_INLINE           static __inline
  #define __NO_RETURN               __attribute__((noreturn))
  #define __USED                    __attribute__((used))
  #define __WEAK                    __attribute__((weak))
  #define __PACKED                  __attribute__((packed, aligned(1)))
  #pragma clang diagnostic push
  #pragma clang diagnostic ignored "-Wpacked"
  struct __attribute__((packed))    T_UINT32_ { uint32_t v; };
  #pragma clang diagnostic pop
  #define __UNALIGNED_UINT32(x)     (((struct T_UINT32_ *)(x))->v)
  #define __AT_ADDR__(x)            __attribute__((section(".ARM.__AT_"#x)))
  #define __AT_ADDR(x)              __AT_ADDR__(x)
  #define __SECTION__(x)            __attribute__((section(#x)))
  #define __SECTION(x)              __SECTION__(x)

  #define osThreadDef_(name, ...)   extern const osThreadDef_t os_thread_def_##name;          \
                                    osThreadDef(name, __VA_ARGS__)
  #define osTimerDef_(name, ...)    extern const osTimerDef_t os_timer_def_##name;            \
                                    extern           uint32_t os_timer_cb_##name[6];          \
                                    osTimerDef(name, __VA_ARGS__)
  #define osMutexDef_(name)         extern const osMutexDef_t os_mutex_def_##name;            \
                                    extern uint32_t           os_mutex_cb_##name[4];          \
                                    osMutexDef(name)
  #define osSemaphoreDef_(name)     extern const osSemaphoreDef_t os_semaphore_def_##name;    \
                                    extern uint32_t               os_semaphore_cb_##name[2];  \
                                    osSemaphoreDef(name)

#else
  #error Unsupported compiler
#endif

#endif  // __USB_COMPILER_H__
